/*
    Project: TCGINA (Keyfile Library/Editor V1.0f)
    Version: V1.15
    File   : kfi.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "kfipriv.h"

int KFI_InitEncryptionData(const KFI_DATA *pd, KFI_ENCRYPTION_DATA *ped,
  const void *pPassword, size_t cbPasswordSize)
{
    unsigned long uOutLen;
    int           iError;

    ped->fValid = FALSE;

    if (cbPasswordSize) {
        if (register_hash(&KFI_HASH_DESC) == -1) {
            return CRYPT_INVALID_HASH;
        }

        uOutLen = KFI_TOTAL_KEY_SIZE;
        if ((iError = pkcs_5_alg2(pPassword, (unsigned)cbPasswordSize,
          pd->rgbSalt, sizeof(pd->rgbSalt), KFI_ITERATION_COUNT,
          find_hash(KFI_HASH_NAME), (unsigned char*) ped, &uOutLen)) !=
          CRYPT_OK) {
             return iError;
        }
        if (uOutLen != KFI_TOTAL_KEY_SIZE) {
            return CRYPT_ERROR;
        }

        ped->fValid = TRUE;

        return CRYPT_OK;
    }

    return CRYPT_OK;
}

#ifdef KFI_DEBUG
extern const DWORD rgdwCRC32Table[256];
BOOL KFI_TestFunc(void)
{
    if (rijndael_test() != CRYPT_OK) {
        return FALSE;
    }
    if (twofish_test() != CRYPT_OK) {
        return FALSE;
    }

    if (calc_crc32("123456789", 9) != 0xcbf43926) {
        return FALSE;
    }
    if (calc_crc32(rgdwCRC32Table, sizeof(rgdwCRC32Table)) != 0x6fcf9e13) {
        return FALSE;
    }

#ifndef KFI_DECRYPTION_ONLY
    if (sha256_test() != CRYPT_OK) {
        return FALSE;
    }
#endif
    if (sha512_test() != CRYPT_OK) {
        return FALSE;
    }

    return TRUE;
}
#endif
