/*
    Project: TCGINA (Keyfile Library/Editor V1.0f)
    Version: V1.15
    File   : kfi.h

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#ifndef __KFI_H_D47DF546F16B490bA6D8523603A5594D
#define __KFI_H_D47DF546F16B490bA6D8523603A5594D

#include <windows.h>

/* Error codes are taken from LibTomCrypt */
enum {
   KFIE_OK=0,
   KFIE_ERROR,
   KFIE_NOP,

   KFIE_INVALID_KEYSIZE,
   KFIE_INVALID_ROUNDS,
   KFIE_FAIL_TESTVECTOR,

   KFIE_BUFFER_OVERFLOW,
   KFIE_INVALID_PACKET,

   KFIE_INVALID_PRNGSIZE,
   KFIE_ERROR_READPRNG,

   KFIE_INVALID_CIPHER,
   KFIE_INVALID_HASH,
   KFIE_INVALID_PRNG,

   KFIE_MEM,

   KFIE_INVALID_ARG = 16
};

#pragma pack(push, 1)

#define KFI_MAX_PASSWORD_LEN        128
#define KFI_MAX_PASSWORD            (KFI_MAX_PASSWORD_LEN+1)

#define KFI_MAX_DESCRIPTION         ((8*16-4)/sizeof(WCHAR))
#define KFI_MAX_HOST_NAME           260
#define KFI_MAX_VOLUME_KEY          1024
#define KFI_MAX_VOLUME_PASSWORD_LEN 128
#define KFI_MAX_VOLUME_PASSWORD     (KFI_MAX_VOLUME_PASSWORD_LEN+1)

typedef struct
{
    /* 8 blocks */
    BYTE  rgbSalt[128];

    /* P1 */
    DWORD dwSignature;
    DWORD dwCRC;
    BYTE  rgbUnusedSpace[8];

    /* P2 */
    WORD  wVersion;
    WORD  wType;
    DWORD dwID;
    BYTE  rgbUnusedSpace2[8];

    /* P3...P10 (8 blocks) */
    BYTE  rgbUnusedSpace3[4];
    WCHAR wszDescription[KFI_MAX_DESCRIPTION];

    /* P11 */
    CHAR  chDriveLetter;
    BYTE  rgbReserved[7];
    BYTE  rgbUnusedSpace4[8];

    /* P12 */
    DWORD dwMountMask;
    DWORD dwMountOptions;
    BYTE  rgbUnusedSpace5[8];

    /* P13...P45 (33 blocks) */
    BYTE  rgbUnusedSpace6[4];
    WCHAR wszHostName[KFI_MAX_HOST_NAME];
    BYTE  rgbUnusedSpace7[4];

    /* P46 */
    WORD  wKeyType;
    WORD  wKeySize;
    WORD  wPasswordType;
    WORD  wPasswordSize;
    BYTE  rgbUnusedSpace8[8];

    /* P47...P111 (65 blocks) */
    BYTE  rgbUnusedSpace9[8];
    BYTE  rgbKey[KFI_MAX_VOLUME_KEY];
    BYTE  rgbUnusedSpace10[8];

    /* P112...P120 (9 blocks) */
    BYTE  rgbUnusedSpace11[4];
    CHAR  aszPassword[KFI_MAX_VOLUME_PASSWORD];
    BYTE  rgbUnusedSpace12[7];
    DWORD dwReserved;

} KFI_DATA, *PKFI_DATA;

#define KFI_CIPHER_KEY_SIZE    32
#define KFI_CIPHER_BLOCK_SIZE  16

#define KFI_SIGNATURE          0xb858b704

/* wType */
#define KFIT_GENERIC           0

/* dwMountOptions */
#define KFIMO_CACHE_PASSWORDS  0x00000001
#define KFIMO_READ_ONLY        0x00000002
#define KFIMO_REMOVABLE        0x00000004
#define KFIMO_SHARED           0x00000008
#define KFIMO_UPDATE_TIMESTAMP 0x00000010

/* wKeyType */
#define KFIKT_GENERIC          0

/* wPasswordType */
#define KFIPT_INCOMPLETE       0x0001
#define KFIPT_COMPLETE         0x0002

typedef struct
{
    BYTE rgbKey1[KFI_CIPHER_KEY_SIZE];
    BYTE rgbKey2[KFI_CIPHER_KEY_SIZE];
    BYTE rgbIV[KFI_CIPHER_BLOCK_SIZE];
    BYTE rgbWhiteningKey[KFI_CIPHER_BLOCK_SIZE];
    BOOL fValid;
} KFI_ENCRYPTION_DATA, *PKFI_ENCRYPTION_DATA;

#pragma pack(pop)

#ifdef __cplusplus
extern "C" {
#endif

BOOL KFI_InitRandomPool(void);
BOOL KFI_DestroyRandomPool(void);

int KFI_Init(KFI_DATA *ppd);
int KFI_InitEncryptionData(const KFI_DATA *ppd, KFI_ENCRYPTION_DATA *ped,
  const void *pPassword, size_t cbPasswordSize);
int KFI_UpdateSalt(KFI_DATA *pwdDest);
int KFI_Encrypt(KFI_DATA *pwdDest, const KFI_DATA *pwdSrc,
  const KFI_ENCRYPTION_DATA *ped);
int KFI_Decrypt(KFI_DATA *pwdDest, const KFI_DATA *pwdSrc,
  const KFI_ENCRYPTION_DATA *ped);
BOOL KFI_Edit(PKFI_DATA pd, void *pPassword, DWORD cbPasswordSize,
  LPCTSTR pszCaption);
BOOL KFI_TestFunc(void);

#ifdef __cplusplus
}
#endif

#ifdef KFI_DEBUG
#  define KFI_Test() KFI_TestFunc()
#else
#  define KFI_Test() (TRUE)
#endif

#endif
