/*
    Project: REG
    Version: V1.2
    File   : REG.C

    Copyright (c) 2005-2006 Author of TCGINA DLL
	
    The program REG may be used, modified or distributed under the terms 
    of the TrueCrypt License Version 2.0 (see file "License.txt").

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/
#include <windows.h>
#include <tchar.h>
#include <stdio.h>

#define VERSION TEXT("1.2")

int GetType(LPTSTR pszType)
{
	const static struct {
		LPTSTR pszName;
		int    iType;
	} Type[] = {
		{ TEXT("REG_SZ"), REG_SZ },
		{ TEXT("REG_MULTI_SZ"), REG_MULTI_SZ },
		{ TEXT("REG_DWORD"), REG_DWORD },
		{ NULL, REG_NONE }
	};
	int i;
	for (i = 0; Type[i].pszName != NULL; i++) {
		if (_tcscmp(pszType, Type[i].pszName) == 0) {
			return Type[i].iType;
		}
	}

	return REG_NONE;
}

LPTSTR GetRoot(LPTSTR pszFullKeyName, PHKEY phKey)
{
	const static struct {
		LPTSTR pszName;
		HKEY   hKey;
	} Root[] = {
		{ TEXT("HKEY_CLASSES_ROOT"), HKEY_CLASSES_ROOT },
		{ TEXT("HKEY_CURRENT_USER"), HKEY_CURRENT_USER },
		{ TEXT("HKEY_LOCAL_MACHINE"), HKEY_LOCAL_MACHINE },
		{ TEXT("HKEY_USERS"), HKEY_USERS },
		{ TEXT("HKCU"), HKEY_CURRENT_USER },
		{ TEXT("HKLM"), HKEY_LOCAL_MACHINE },
		{ NULL, NULL }
	};
	int i;
	LPTSTR pszKeyName = _tcschr(pszFullKeyName, TEXT('\\'));

	if (pszKeyName == NULL) return NULL;
	*pszKeyName++ = TEXT('\0');

	for (i = 0; Root[i].pszName != NULL; i++) {
		if (_tcscmp(pszFullKeyName, Root[i].pszName) == 0) {
			if (phKey) {
				*phKey = Root[i].hKey;
			}
			return pszKeyName;
		}
	}

	return NULL;
}

BOOL RegSetString(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName, 
  DWORD dwType, LPCTSTR pszValueData)
{
	HKEY hKey;
	BOOL  fResult = FALSE;

	if (RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0, 
	  KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) { 
		return FALSE;
	}
	if (RegSetValueEx(hKey, pszValueName, 0, dwType, (const BYTE*)pszValueData, 
	  (DWORD)(_tcslen(pszValueData)+1)*sizeof(TCHAR)) == ERROR_SUCCESS) {
		fResult = TRUE;
	}
	RegCloseKey(hKey);

	return fResult;
}

BOOL RegSetInteger(HKEY hRootKey, LPCTSTR pszKeyName, LPCTSTR pszValueName, 
  DWORD dwData)
{
	HKEY hKey;
	BOOL fResult = FALSE;

	if (RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0, 
	  KEY_WRITE, NULL, &hKey, NULL) != ERROR_SUCCESS) { 
		return FALSE;
	}
	if (RegSetValueEx(hKey, pszValueName, 0, REG_DWORD, (const BYTE*)&dwData, 
	  sizeof(DWORD)) == ERROR_SUCCESS) {
		fResult = TRUE;
	}
	RegCloseKey(hKey);

	return fResult;
}

BOOL RegRemoveKey(HKEY hRootKey, LPCTSTR pszKeyName)
{
    HKEY     hKey;
	LONG     lResult;
    TCHAR    szSubKeyName[MAX_PATH];
    DWORD    dwSize;
    FILETIME ftLastWriteTime;

	if (RegDeleteKey(hRootKey, pszKeyName) == ERROR_SUCCESS) {
		return TRUE;
	}

	lResult = RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_READ, &hKey);
	if (lResult != ERROR_SUCCESS) {
		if (lResult == ERROR_FILE_NOT_FOUND) {
			return TRUE;
		}
		SetLastError(lResult);
		return FALSE;
    }

    dwSize = sizeof(szSubKeyName)/sizeof(TCHAR);
    while (RegEnumKeyEx(hKey, 0, szSubKeyName, &dwSize, NULL,
	  NULL, NULL, &ftLastWriteTime) == ERROR_SUCCESS) {
		if (!RegRemoveKey(hKey, szSubKeyName)) {
			DWORD dwLastError = GetLastError();
			RegCloseKey(hKey);
			SetLastError(dwLastError);
			return FALSE;
		}
		dwSize = sizeof(szSubKeyName)/sizeof(TCHAR);
	}
    RegCloseKey(hKey);

	lResult = RegDeleteKey(hRootKey, pszKeyName) == ERROR_SUCCESS;
	if (lResult == ERROR_SUCCESS) {
		return TRUE;
	}
	SetLastError(lResult);
	return FALSE;
}

BOOL RegRemoveValue(HKEY hRootKey, LPCTSTR pszKeyName, 
  LPCTSTR pszValueName)
{
	HKEY hKey;
	LONG lResult;

	lResult = RegOpenKeyEx(hRootKey, pszKeyName, 0, KEY_WRITE, &hKey);
	if (lResult != ERROR_SUCCESS) {
		SetLastError(lResult);
		return FALSE;
	}

	lResult = RegDeleteValue(hKey, pszValueName);
	if (lResult != ERROR_SUCCESS) {
		RegCloseKey(hKey);
		SetLastError(lResult);
		return FALSE;
	}
	RegCloseKey(hKey);

	return TRUE;
}

LPTSTR ConvertString(LPCTSTR pszSrc, BOOL fMultiString, PDWORD pcbSize)
{
	DWORD  cbSize;
	LPTSTR pszDestBuffer;
	LPTSTR pszDest;

	cbSize = (DWORD)(_tcslen(pszSrc) + 1 + (fMultiString ? 1 : 0)) * 
	  sizeof(TCHAR);
	pszDestBuffer = (LPTSTR) LocalAlloc(LMEM_FIXED | LMEM_ZEROINIT, cbSize);
	if (pszDestBuffer == NULL) {
		return NULL;
	}
	pszDest = pszDestBuffer;

	while (*pszSrc) {
		if (pszSrc[0] == '&' && pszSrc[1] == '#') {
			LPTSTR psz;
			ULONG ul = _tcstoul(pszSrc+2, &psz, 10);
			if (*psz == ';' && ul < 0x10000)  {
				*pszDest++ = (TCHAR) ul;
				pszSrc = psz + 1;
				continue;
			}
		}
		*pszDest++ = *pszSrc++;
	}
	if (fMultiString) {
		pszDest += 2;
	} else {
		pszDest = pszDestBuffer + _tcslen(pszDestBuffer) + 1;
	}

	if (pcbSize) {
		*pcbSize = (DWORD)((PBYTE)pszDest - (PBYTE)pszDestBuffer);
	}

	return pszDestBuffer;
}

int _tmain(int argc, _TCHAR *argv[])
{
	if (argc < 2) goto display_syntax;

	if (_tcsicmp(argv[1], TEXT("SET")) == 0) {
		HKEY   hRootKey;
		int    iType = GetType(argv[2]);
		LPTSTR pszKeyName = GetRoot(argv[3], &hRootKey);
		HKEY   hKey;
		LONG   lResult;

		if (argc != 6) goto display_syntax;
		
		if (iType == REG_NONE || pszKeyName == NULL) goto display_syntax;

		lResult = RegCreateKeyEx(hRootKey, pszKeyName, 0, NULL, 0, 
		  KEY_WRITE, NULL, &hKey, NULL);
		if (lResult != ERROR_SUCCESS) return lResult;

		if (iType == REG_SZ) {
			DWORD cbValueData;
			LPTSTR pszValueData = ConvertString(argv[5], FALSE, &cbValueData);
			if (pszValueData == NULL) {
				return ERROR_NOT_ENOUGH_MEMORY;
			} 
			lResult = RegSetValueEx(hKey, argv[4], 0, REG_SZ, 
			  (const BYTE*)pszValueData, cbValueData);
			LocalFree(pszValueData);
			if (lResult != ERROR_SUCCESS) return lResult;
		} else if (iType == REG_MULTI_SZ) {
			DWORD cbValueData;
			LPTSTR pszValueData = ConvertString(argv[5], TRUE, &cbValueData);
			if (pszValueData == NULL) {
				return ERROR_NOT_ENOUGH_MEMORY;
			} 
			lResult = RegSetValueEx(hKey, argv[4], 0, REG_MULTI_SZ, 
			  (const BYTE*)pszValueData, cbValueData);
			LocalFree(pszValueData);
			if (lResult != ERROR_SUCCESS) return lResult;
		} else {
			DWORD dwData = _tcstoul(argv[5], NULL, 0);
			lResult = RegSetValueEx(hKey, argv[4], 0, REG_DWORD, 
			  (const BYTE*)&dwData, sizeof(DWORD));
			if (lResult != ERROR_SUCCESS) return lResult;
		} 
		RegCloseKey(hKey);
	} else if (_tcsicmp(argv[1], TEXT("DELETE")) == 0) {
		HKEY   hRootKey;
		LPTSTR pszKeyName = GetRoot(argv[2], &hRootKey);
		if (argc == 3) {
			if (!RegRemoveKey(hRootKey, pszKeyName)) {
				return GetLastError();
			}
		} else if (argc == 4) {
			if (!RegRemoveValue(hRootKey, pszKeyName, argv[3])) {
				return GetLastError();
			}
		} else {
			goto display_syntax;
		}
	} else {
		goto display_syntax;
	}

	return NO_ERROR;

display_syntax:
    _putts(
        TEXT("REG V") VERSION TEXT("\n")
        TEXT("\n")
        TEXT("SYNTAX 1: REG SET Type Key Value Data\n")
        TEXT("\n")
		TEXT("  Type  : Value type (REG_SZ, REG_MULTI_SZ or REG_DWORD)\n")
		TEXT("  Key   : Registry key which must start either with\n")
		TEXT("          HKEY_LOCAL_MACHINE or HKEY_CURRENT_USER\n")
		TEXT("  Value : Value name\n")
		TEXT("  Data  : Value data\n")
		TEXT("\n")
		TEXT("  The registry value is correspondingly set.\n")
		TEXT("  The HTML escape sequence \"&#<d>;\" can be used to define Unicode characters,\n")
		TEXT("  where <d> is a placeholder for the decimal Unicode number.\n")
		TEXT("\n")
        TEXT("SYNTAX 2: REG DELETE Key [Value]\n")
        TEXT("\n")
		TEXT("  The registry key or value is deleted.\n")
        TEXT(""));

    return ERROR_INVALID_PARAMETER;
}
