/*
    Project: TCGINA
    Version: V1.15
    File   : tcgmisc.c

    Copyright (c) 2005-2006 Author of TCGINA DLL. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.2.4 (MingW32)

mQGiBEI3cVwRBADCRqJ1+AKeMw4BagfF9yNdobBBJXTFQ0R3RJHRAOoKMrT5rxMn
zjRvN9SugcTAftMRVTmsHVZcve6jO2yLHJLg2QgtHiRKdbLV9MTrzYtxXUL1Ez0A
3Mq6ww/h7EdwG5VlPX8uw++Y2aX2UHvMgRbImcLDz+/8qT1FR6dvujjyhwCgmUR9
/4+H8QkCw7M2ekFsODzSBzsEAIwTWuqYfwPRISyGTL/ajZC2WVxLNCC1NmvxzRMe
v/Mg344nxOrMG6MNTNYJdq/3dkiWBzXcpy5zWAVMxAiMDhKSytdk5mdvPI69wZtS
UZ92qXxvm/6fPjIoFy1hSII0LExLuS5p0MMZ7yFFEjZ4CQG/fPE04aUF0zAVgQQ8
OOjJA/0U+8ZlJnPF5cMfvODlOzjfG+LFhWS/t4NhJ12CWeOyn8+enAiXQsKyKyll
JtE9zf8aQKhfS1KJ/ZLZphmnN5LqQLOi/HVsAIK4/KPRrv7M4Z+atChLbmvvR31I
AcTOoDqxj4d8D8yiV8i+ZfLAXREPVLXc3c/W5gBrYwyp5bmpG7QUQXV0aG9yIG9m
IFRDR0lOQSBETEyIXgQTEQIAHgUCQjdxXAIbAwYLCQgHAwIDFQIDAxYCAQIeAQIX
gAAKCRD/R4xyQJdnzk9iAJ9rTCyGTUioEOJEU2w779rVzXoUkQCeO8zdw6QakpiA
uc2XmzGb6JscCEA=
=2DjS
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tcgpriv.h"
#include <Winnetwk.h>
#include <tchar.h>

#ifdef TC_DEBUG
static void TC_DbgPrintMultiString(LPTSTR pszFormat, LPCTSTR pszStrings)
{
    while (*pszStrings) {
        TC_DbgPrintf(pszFormat, pszStrings);
        pszStrings += _tcslen(pszStrings) + 1;
    }
}
#endif

#define ST_GENERAL        0
#define ST_USER           1
#define ST_REMOTE_SESSION 2

void WINAPI TCG_ReadSettings(PTCG_SETTINGS pSettings,
  LPCTSTR pszUserName)
{
    HKEY  hKey;
    TCHAR szRegKeyName[MAX_PATH];
    int   iType;

    TC_Assert(pSettings);

    if (pszUserName == NULL) {
        iType = ST_GENERAL;
    } else {
        iType = (_tcschr(pszUserName, TEXT('\\')) == NULL) ?
          ST_USER : ST_REMOTE_SESSION;
    }

    if (iType == ST_USER) {
        TCG_ReadSettings(pSettings, NULL);
    }

    _tcscpy(szRegKeyName, TCG_REGKEY_NAME);
    if (iType == ST_GENERAL) {
        /* Initialize settings with default values */
        TC_ZeroMemory(pSettings, sizeof(TCG_SETTINGS));
    } else {
        if (!TC_AppendString(szRegKeyName, sizeof(szRegKeyName)/sizeof(TCHAR),
          TEXT("\\"))) {
            TC_DbgPutsA("Cannot append backslash to registry key name!\n");
            return;
        }
        if (!TC_AppendString(szRegKeyName, sizeof(szRegKeyName)/sizeof(TCHAR),
          pszUserName)) {
            TC_DbgPutsA("Cannot append user name to registry key name!\n");
            return;
        }
    }

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKeyName, 0, KEY_READ, &hKey) ==
      ERROR_SUCCESS) {
#ifdef SUPPORT_KEYFILES
        DWORD dw;
#endif
        TC_ReadRegistryInteger(hKey, TEXT("TCOptions"),
          &pSettings->dwTCOptions);
        TC_ReadRegistryInteger(hKey, TEXT("TCMountOptions"),
          &pSettings->dwTCMountOptions);
        TC_ReadRegistryInteger(hKey, TEXT("TCPasswordTimeout"),
          &pSettings->dwTCPasswordTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("TCPasswordTotalTimeout"),
          &pSettings->dwTCPasswordTotalTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("TCKeyFileTimeout"),
          &pSettings->dwTCKeyFileTimeout);

        TC_ReadRegistryStringList(hKey, TEXT("TCPath"), pSettings->szTCPath,
          sizeof(pSettings->szTCPath)/sizeof(TCHAR));
        if (TC_ReadRegistryString(hKey, TEXT("TCFileName"),
          pSettings->szTCFileName,
          sizeof(pSettings->szTCFileName)/sizeof(TCHAR))) {
            if (*pSettings->szTCFileName) {
                pSettings->fTCFileNameAvailable = TRUE;
            }
        }
        TC_ReadRegistryString(hKey, TEXT("TCDrive"), pSettings->szTCDrive,
          sizeof(pSettings->szTCDrive)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("TCFirstAutoMountDrive"),
          pSettings->szTCFirstAutoMountDrive,
          sizeof(pSettings->szTCFirstAutoMountDrive)/sizeof(TCHAR));
        TC_ReadRegistryStringList(hKey, TEXT("TCKeyFileNames"),
          pSettings->szTCKeyFileNames,
          sizeof(pSettings->szTCKeyFileNames)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("TCKeyFileDrives"),
          pSettings->szTCKeyFileDrives,
          sizeof(pSettings->szTCKeyFileDrives)/sizeof(TCHAR));

#ifdef SUPPORT_TCPASSWORD
        TC_ReadRegistryString(hKey, TEXT("TCPassword"),
          pSettings->szTCPassword,
          sizeof(pSettings->szTCPassword)/sizeof(TCHAR));
#endif

#ifdef SUPPORT_KEYFILES
        TC_ReadRegistryString(hKey, TEXT("KFFileName"),
          pSettings->szKFFileName,
          sizeof(pSettings->szKFFileName)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("KFDrives"),
          pSettings->szKFDrives,
          sizeof(pSettings->szKFDrives)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("KFPassword"),
          pSettings->szKFPassword,
          sizeof(pSettings->szKFPassword)/sizeof(TCHAR));
        if (TC_ReadRegistryInteger(hKey, TEXT("KFFileOffset"),
          &dw)) {
            pSettings->liKFFileOffset.LowPart = dw;
        }
        if (TC_ReadRegistryInteger(hKey, TEXT("KFFileOffsetHigh"),
          &dw)) {
            pSettings->liKFFileOffset.HighPart = dw;
        }
        TC_ReadRegistryInteger(hKey, TEXT("KFFileSize"),
          &pSettings->dwKFFileSize);
        TC_ReadRegistryInteger(hKey, TEXT("KFID"),
          &pSettings->dwKFID);
        TC_ReadRegistryInteger(hKey, TEXT("KFNoPasswordDialog"),
          &pSettings->dwKFNoPasswordDialog);
        TC_ReadRegistryInteger(hKey, TEXT("KFMediumTimeout"),
          &pSettings->dwKFMediumTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("KFPasswordTimeout"),
          &pSettings->dwKFPasswordTimeout);
        TC_ReadRegistryInteger(hKey, TEXT("KFPasswordTotalTimeout"),
          &pSettings->dwKFPasswordTotalTimeout);
#endif

#ifdef SUPPORT_NETWORK
        TC_ReadRegistryString(hKey, TEXT("NDLocalName"),
          pSettings->szNDLocalName,
          sizeof(pSettings->szNDLocalName)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("NDRemoteName"),
          pSettings->szNDRemoteName,
          sizeof(pSettings->szNDRemoteName)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("NDUserName"),
          pSettings->szNDUserName,
          sizeof(pSettings->szNDUserName)/sizeof(TCHAR));
        TC_ReadRegistryString(hKey, TEXT("NDPassword"),
          pSettings->szNDPassword,
          sizeof(pSettings->szNDPassword)/sizeof(TCHAR));
#endif

#ifdef SUPPORT_SERVICES
        TC_ReadRegistryStringList(hKey, TEXT("TCStartServices"),
          pSettings->szTCStartServices,
          sizeof(pSettings->szTCStartServices)/sizeof(TCHAR));
        TC_ReadRegistryStringList(hKey, TEXT("TCStopServices"),
          pSettings->szTCStopServices,
          sizeof(pSettings->szTCStartServices)/sizeof(TCHAR));
#endif

        RegCloseKey(hKey);
    }

    pSettings->dwTCOptions      &= TC_DISPLAY_PASSWORD |
                                   TC_IGNORE_USER_ACCOUNT_PASSWORD |
                                   TC_AUTO_REPAIR_PROFILE_PATH;
    pSettings->dwTCMountOptions &= TC_CACHE_PASSWORDS |
                                   TC_READ_ONLY |
                                   TC_REMOVABLE |
                                   TC_SHARED |
                                   TC_UPDATE_TIMESTAMPS;
    pSettings->dwTCMountOptions |= TC_PERSISTENT;

    if (_tcschr(pSettings->szTCFileName, TEXT('\\'))) {
        *pSettings->szTCPath = TEXT('\0');
    } else if (*pSettings->szTCPath == TEXT('\0')) {
        TC_GetProfilesDirectory(pSettings->szTCPath,
          sizeof(pSettings->szTCPath)/sizeof(TCHAR));
    }

    if (*pSettings->szTCFileName == TEXT('\0')) {
        _tcscpy(pSettings->szTCFileName, TEXT("%USERNAME%.tc"));
    }
    if (*pSettings->szTCDrive == TEXT('\0')) {
        _tcscpy(pSettings->szTCDrive, TEXT("U:\\"));
    }

    if (iType == ST_USER) {

        if (GetSystemMetrics(SM_REMOTESESSION)) {

            TCHAR szRemoteName[MAX_PATH];

            if (TC_CopyString(szRemoteName, sizeof(szRemoteName)/sizeof(TCHAR),
              pszUserName) &&
              TC_AppendString(szRemoteName, sizeof(szRemoteName)/sizeof(TCHAR),
              TEXT("\\RemoteSession"))) {
                TCG_ReadSettings(pSettings, szRemoteName);
            }
        }

        TC_ReplaceString(pSettings->szTCPath,
          sizeof(pSettings->szTCPath)/sizeof(TCHAR),
          TEXT("%USERNAME%"), pszUserName);

        TC_ReplaceString(pSettings->szTCFileName,
          sizeof(pSettings->szTCFileName)/sizeof(TCHAR),
          TEXT("%USERNAME%"), pszUserName);

#ifdef TC_DEBUG
        TC_DbgPrintf(TEXT("Settings for user \"%s\":\n"), pszUserName);
        TC_DbgPrintfA(    "  TCOptions               = 0x%08x\n"
                          "  TCMountOptions          = 0x%08x\n"
                          "  TCPasswordTimeout       = %u\n"
                          "  TCKeyFileTimeout        = %u\n",
                          pSettings->dwTCOptions,
                          pSettings->dwTCMountOptions,
                          pSettings->dwTCPasswordTimeout,
                          pSettings->dwTCKeyFileTimeout);
        TC_DbgPrintMultiString(
                     TEXT("  TCPath                  = %s\n"),
                     pSettings->szTCPath);
        if (pSettings->fTCFileNameAvailable) {
            TC_DbgPrintf(TEXT("  TCFileName              = %s\n"),
              pSettings->szTCFileName);
        } else {
            TC_DbgPrintf(TEXT("  TCFileName              = n/a (%s)\n"),
              pSettings->szTCFileName);
        }
        TC_DbgPrintf(TEXT("  TCDrive                 = %s\n"),
          pSettings->szTCDrive);
        TC_DbgPrintf(TEXT("  TCFirstAutoMountDrive   = %s\n"),
          pSettings->szTCFirstAutoMountDrive);
        TC_DbgPrintf(TEXT("  TCKeyFileDrives         = %s\n"),
          pSettings->szTCKeyFileDrives);
        TC_DbgPrintMultiString(
                     TEXT("  TCKeyFileName           = %s\n"),
                     pSettings->szTCKeyFileNames);

#ifdef SUPPORT_KEYFILES
        TC_DbgPrintf(TEXT("  KFFileName              = %s\n"),
          pSettings->szKFFileName);
        TC_DbgPrintf(TEXT("  KFDrives                = %s\n"),
          pSettings->szKFDrives);
        TC_DbgPrintfA(    "  KFPassword              = (%s)\n",
            *pSettings->szKFPassword ? "non-empty" : "empty");
        TC_DbgPrintfA(    "  KFFileOffset            = 0x%08x\n"
                          "  KFFileOffsetHigh        = 0x%08x\n"
                          "  KFFileSize              = %u\n"
                          "  KFID                    = 0x%08X\n"
                          "  KFNoPasswordDialog      = %u\n"
                          "  KFMediumTimeout         = %u\n"
                          "  KFPasswordTimeout       = %u\n",
                          pSettings->liKFFileOffset.LowPart,
                          pSettings->liKFFileOffset.HighPart,
                          pSettings->dwKFFileSize,
                          pSettings->dwKFID,
                          pSettings->dwKFNoPasswordDialog,
                          pSettings->dwKFMediumTimeout,
                          pSettings->dwKFPasswordTimeout);
#endif

#ifdef SUPPORT_NETWORK
        TC_DbgPrintf(TEXT("  NDLocalName             = %s\n"),
          pSettings->szNDLocalName);
        TC_DbgPrintf(TEXT("  NDRemoteName            = %s\n"),
          pSettings->szNDRemoteName);
        TC_DbgPrintf(TEXT("  NDUserName              = %s\n"),
          pSettings->szNDUserName);
        TC_DbgPrintfA(    "  NDPassword              = (%s)\n",
            *pSettings->szNDPassword ? "non-empty" : "empty");
        TC_DbgPrintMultiString(
                     TEXT("  TCStopServices          = (%s)\n"),
                     pSettings->szTCStopServices);
        TC_DbgPrintMultiString(
                     TEXT("  TCStartServices         = (%s)\n"),
                     pSettings->szTCStartServices);
#endif

#endif
    }
}

/*-----------------------------------------------------------------------------
// Network functions                                                         */

#ifdef SUPPORT_NETWORK

DWORD WINAPI TCG_MapNetworkDrive(LPTSTR pszLocalName,
  LPTSTR pszRemoteName, LPCTSTR pszUserName, LPCTSTR pszPassword)
{
    /*-------------------------------------------------------------------------
    // References:
    //
    // http://support.microsoft.com/default.aspx?scid=kb;en-us;Q180362
    //-----------------------------------------------------------------------*/

    TCHAR       szDomainAndUserName[MAX_PATH];
    NETRESOURCE nr;
    DWORD       dwErrorCode;

    TC_Assert(pszLocalName);
    TC_Assert(pszRemoteName);
    TC_Assert(pszUserName);
    TC_Assert(pszPassword);

    if (*pszLocalName == TEXT('\0')) return NO_ERROR;

    if (_tcschr(pszUserName, TEXT('\\')) == 0) {
        if (TC_CopyString(szDomainAndUserName, MAX_PATH, pszRemoteName+2)) {
            LPTSTR pszTemp = _tcschr(szDomainAndUserName, TEXT('\\'));
            if (pszTemp) {
                pszTemp[1] = TEXT('\0');
                if (TC_AppendString(szDomainAndUserName, MAX_PATH,
                  pszUserName)) {
                    pszUserName = szDomainAndUserName;
                }
            }
        }
    }

    TC_ZeroMemory(&nr, sizeof(nr));
    nr.dwType = RESOURCETYPE_DISK;
    nr.lpLocalName = pszLocalName;
    nr.lpRemoteName = pszRemoteName;

    dwErrorCode = WNetAddConnection2(&nr, pszPassword, pszUserName, 0);
    if (dwErrorCode != NO_ERROR) {
        TC_DbgPrintf(
          TEXT("WNetAddConnection2(%s, %s, %s) failed (ErrorCode=%d)\n"),
          pszLocalName, pszRemoteName, pszUserName, dwErrorCode);
    }

    return dwErrorCode;
}

DWORD WINAPI TCG_UnmapNetworkDrive(LPCTSTR pszLocalName)
{
    DWORD dwErrorCode;

    TC_Assert(pszLocalName);

    if (*pszLocalName == TEXT('\0')) return NO_ERROR;

    dwErrorCode = WNetCancelConnection2(pszLocalName, 0, TRUE);
    if (dwErrorCode != NO_ERROR) {
        TC_DbgPrintf(
          TEXT("WNetCancelConnection2(%s) failed (ErrorCode=%d)\n"),
          pszLocalName, dwErrorCode);
    }

    return dwErrorCode;
}

#endif
